// Author 		: F. de la Parra
// Date			: March, 2019

public class Bus extends Vehicle implements Scheduled {
	
	// Bus attributes
	private String bodyType;       // DOUBLE-DECKER, SHUTTLE, COACH
	private int passengers;        // Number of passengers
	private double luggageLoad;    // Maximum luggage compartment load [Kgs]
	
	// enum to identify the acceptable bus types.
	enum busSize{SHUTTLE, COACH, DOUBLE_DECKER, ARTICULATED};
	
	// Default constructor.
	public Bus() {
		super();
		bodyType = "";
		passengers = 0;
		luggageLoad = 0;
	}
	
	// Full parameter constructor.
	public Bus(String busType, int capacity, double luggage) {
		// At this point, for example, we can run the default constructor of
		// the 'Vehicle' base class to partially initialize a 'Bus' object.
		super();
		
		// The following code completes initialization of a 'Bus' object.
		try {
			// Convert 'string' busType to 'enumerated value' tempSize.
			busSize tempSize = busSize.valueOf(busType.toUpperCase());
		
			// Initialize valid Bus object for valid busType.
			bodyType = busType;
			passengers = capacity;
			luggageLoad = luggage;
		}
		catch (IllegalArgumentException e){
			System.out.println("'" + busType + "' is an invalid bus size! Program aborted!");
			System.exit(0);
		}
	}
	
	// Accessor methods.
	
	public String getBodyType() {
		return bodyType;
	}
	
	public int getPassengers() {
		return passengers;
	}
	
	public double getLuggageLoad() {
		return luggageLoad;
	}
	
	public String getBaseAttributes() {
		// Base class has private attributes. Use 'Vehicle' accessor methods.
		return "identifier = " + super.getIdentifier() + "\n" +
			   "year = " + super.getYear() + "\n" +
			   "make = " + super.getMake() + "\n" +
			   "fuelType = " + super.getFuelType();	
		   
		// Base class has protected attributes. We can use 'Vehicle' attributes
		// directly.
	/*	return "identifier = " + super.identifier + "\n" +
			   "year = " + super.year + "\n" +
			   "make = " + super.make + "\n" +
			   "fuelType = " + super.fuelType;
	*/
	
	}
	
	// Mutator methods
	
	public void setBodyType(String newBodyType) {
		bodyType = newBodyType;
	}
	
	public void setPassengers(int newPassengers) {
		passengers = newPassengers;
	}
	
	public void setLuggageLoad(double newLuggageLoad) {
		luggageLoad = newLuggageLoad;
	}
	
	// BEHAVIOUR METHODS
	
	// Basic methods.
	public String toString() {
		return super.toString();
	}
	
	public String toString(String busType) {
		return busType + " parameters are:\n" +
			   "bodyType = " + bodyType + "\n" +
			   "passengers = " + passengers + "\n" +
			   "luggageLoad = " + luggageLoad;
	}
	
	// Application behaviour methods.
	
	public boolean addItem(int id, Vehicle schedItem) {
		System.out.println("Adding bus " + id + " to scheduled items.");
		return true;
	}

	public boolean deleteItem(int id) {
		System.out.println("Deleting bus " + id + " from scheduled items." + id);
		return true;
	}
	
}