// Author 		: F. de la Parra
// Date			: March, 2019
// Description  : This program executes short demos of concepts in Java 'Generics'.
// Version      : 1.0

import java.util.Scanner;
import java.util.ArrayList;
import java.util.InputMismatchException;


public class GenericsDemo {
	// Enumeration to label tests. 
	enum demo {NONE, ARRAYLIST, COMPARISON, DEMO_C, EXIT};
	
	public static void main(String[] args) {
		 
		// Create a 'GenericsDemo' object.
		GenericsDemo oneInstance = new GenericsDemo();
		
		// Create Scanner object 'menuSelect' for input from keyboard. 
		Scanner menuSelect = new Scanner(System.in);
		
		demo selector = demo.NONE;
		int menuItem = 0;
		
		while (!selector.equals(demo.EXIT)) {
			// Display menu
			System.out.println("\n*** Generics Demo ***");
			System.out.println("1 - ArrayList<T>");
			System.out.println("9 - Exit");
			System.out.print("> ");
			
			// Input a menu selection.
			try {
				menuItem = menuSelect.nextInt();
				
				selector = oneInstance.toDemoEnum(menuItem);
				if (!selector.equals(demo.EXIT))
					System.out.println("\n" + selector + " test selected\n");
				
				switch (selector) {
						
					case ARRAYLIST:
						oneInstance.runArrayList();
						break;
						
				}
			}
			catch(InputMismatchException e) {
				System.out.println("Invalid selection. Try again!");
				menuSelect.next();
				selector = demo.NONE;
			}
		}
	}
	
	// Methods for running specific tests on 'generics'.
	
	// Tests on the 'ArrayList<T>' class.
	private void runArrayList() {
		// Methods of ArrayList: get, add, remove, removeRange, clear, indexOf, isEmpty,
		// 						 ensureCapacity, size, toArray, clone, equals.
		
		ArrayList<String> names = new ArrayList<String>(); // Initial capacity of 10.
		System.out.println("Size of 'names' ArrayList = " + names.size());		
	
		names.add("John");
		names.add("Lisa");
		names.add("Mary");
		names.add(1, "Andrew");
		System.out.println("Second name on the list of names: " + names.get(1));
		names.remove(0);	
		System.out.println("Second name on the list of names: " + names.get(1) + "\n");	
		System.out.println("All names:");
		for (String item : names)
			System.out.println(item);
		
		ArrayList<Applicant> admission = new ArrayList<Applicant>(100); // Initial capacity of 100.
		System.out.println("\nSize of 'admission' ArrayList = " + admission.size());
		admission.add(new Applicant());
		admission.add(new Applicant(123456, 85, "Patrick Doe"));
		admission.add(new Applicant(13579, 96, "David Hume"));
		admission.add(new Applicant(124816, 99, "John Locke"));
		
		System.out.println("\n" + admission.get(0).toString());
		System.out.println("\n" + admission.get(1).toString());
		
		admission.set(0, new Applicant(112233, 92, "Francis Bacon"));
		System.out.println("\n" + admission.get(0).toString());
		
	}
	
	// Tests in DemoC.
	private void runDemoC(){
		
		System.out.println("Running demo C!");

	}
	
	// Auxiliary methods.
	
	private demo toDemoEnum(int selection) {
		demo selected = demo.NONE;
		if (selection == 1) 
			selected = demo.ARRAYLIST;
		else if (selection == 9)
			selected = demo.EXIT;
		return selected;
	}
} 