// Author 		: F. de la Parra
// Date			: March, 2019
// Description  : This program checks and reports on several class and interface
//				  constructs that implement class inheritance concepts.

import java.util.Scanner;
import java.util.InputMismatchException;

public class TestHierarchy {
	// Enumeration to label tests. 
	enum test {NONE, OBJECT, VEHICLE, BUS, TAXI, TRUCK, PERSON, DRIVER,
			   CONTROLLER, PASSENGER, SCHEDULED, IDENTIFIABLE, EXIT};
	
	public static void main(String[] args) {
		 
		// Create a 'TestHierarchy' object.
		TestHierarchy oneInstance = new TestHierarchy();
		
		// Create Scanner object 'menuSelect' for input from keyboard. 
		Scanner menuSelect = new Scanner(System.in);
		
		test selector = test.NONE;
		int menuItem = 0;
		
		while (!selector.equals(test.EXIT)) {
			// Display menu
			System.out.println("\n*** Test Class Hierarchies ***");
			System.out.println("1 - Object Class");
			System.out.println("2 - Vehicle Class");
			System.out.println("3 - Bus Class");
			System.out.println("4 - Person Class");
			System.out.println("5 - Driver Class");
			System.out.println("6 - Passenger Class");
			System.out.println("7 - Identifiable Interface");
			System.out.println("9 - Exit");
			System.out.print("> ");
			
			// Input a menu selection.
			try {
				menuItem = menuSelect.nextInt();
				
				selector = oneInstance.toTestEnum(menuItem);
				if (!selector.equals(test.EXIT))
					System.out.println("\n*** " + selector + " test selected ***\n");
				
				switch (selector) {
					case OBJECT:
						// Check basic methods in object
						oneInstance.testObject();
						break;
						
					case VEHICLE:
						oneInstance.testVehicle();
						break;
						
					case BUS:
						oneInstance.testBus();
						break;
						
					case PERSON:
						// Check basic methods in object
						oneInstance.testPerson();
						break;
						
					case DRIVER:
						oneInstance.testDriver();
						break;
						
					case PASSENGER:
						oneInstance.testPassenger();
						break;
						
					case IDENTIFIABLE:
						oneInstance.testIdentifiable();
					break;
				}
			}
			catch(InputMismatchException e) {
				System.out.println("Invalid selection. Try again!");
				menuSelect.next();
				selector = test.NONE;
			}
		}
	}
	
	// Methods for running specific tests on a hierarchy of classes.
	
	// Tests on the 'Object' class.
	private void testObject() {
		// Check toString and hashCode methods in class Object.
		Object myObject = new Object();
		System.out.print("Output of toString() in class 'Object':  ");
		System.out.println(myObject.toString());
		
		System.out.print("\nOutput of hashCode() in class 'Object':  ");
		System.out.println(myObject.hashCode());
	}
	
	// Tests on the 'Vehicle' base class.
	private void testVehicle(){
		Vehicle oneVehicle = new Vehicle();
		System.out.println("Vehicle object created with default constructor:");
		System.out.println(oneVehicle.toString());
		
		System.out.println("\nVehicle object created with full parameter constructor:");
		Vehicle otherVehicle = new Vehicle(12345, 2016, "Tesla", "electric");
		System.out.println(otherVehicle.toString());
		
		System.out.println("\nOutput of comparing both Vehicle objects is " + 
						   oneVehicle.equals(otherVehicle));
	}
	
	// Tests on the 'Bus' derived class.
	private void testBus(){
		Vehicle theVehicle = new Vehicle();
		Truck theTruck = new Truck();
		Bus theBus = new Bus("double_decker", 96, 1200);
		
		System.out.println("Attributes of the base class (Vehicle):");
		System.out.println(theBus.getBaseAttributes());
		
		System.out.println("\nAttributes of the derived class (Truck):");
		System.out.println("bodyType = " + theBus.getBodyType() + "\n" +
						   "passengers = " + theBus.getPassengers() + "\n" +
						   "luggageLoad = " + theBus.getLuggageLoad() + "\n");
						   
		System.out.println("\nbase class (Vehicle) toString() output:");
		System.out.println(theBus.toString());
		
		System.out.println("\noverload of toString() in derived class (Bus) output:");
		System.out.println(theBus.toString("Big Bus"));
		
		System.out.println("\ngetClass() identified for Truck object:");
		System.out.println(theBus.getClass());
		
		System.out.println("\n'Truck instanceof Vehicle' identifies objects of the same class");
		if (theTruck instanceof Vehicle)
			System.out.println("Objects are of the same class type");
		else
			System.out.println("Objects are not of the same class type");
	}
	
	// Tests on the 'Person' base class.
	private void testPerson(){
		//Person testPerson = new Person();
	}
	
	// Tests on the 'Driver' derived class.
	private void testDriver(){
		Driver newDriver = new Driver(12345678, "A");
		System.out.println("New driver:");
		System.out.println("Driver identification is " + newDriver.getUniqueId());
		System.out.println(newDriver.toString());
		
		// Update driver identification to anonymous.
		newDriver.updateIdType("anonymous");
		
		System.out.println("\nAfter setting driver id to 'anonymous'");
		System.out.println("Driver identification is " + newDriver.getUniqueId());
		System.out.println(newDriver.toString());
	}
	
	// Tests on the 'Passenger' derived class.
	private void testPassenger(){
		Passenger passengerObject = new Passenger(12345678, "identified", "casual");
		System.out.println("New passenger:");
		System.out.println("uniqueId = " + passengerObject.getUniqueId() +
						   "\nidType = " + passengerObject.getIdType() +
						   "\nusage = " + passengerObject.getUsage());

		// Update driver identification to anonymous.						   
		passengerObject.updateIdType("anonymous");
		
		System.out.println("\nAfter setting passenger id to 'anonymous'");
		System.out.println("uniqueId = " + passengerObject.getUniqueId() +
						   "\nidType = " + passengerObject.getIdType() +
						   "\nusage = " + passengerObject.getUsage());
	}
	
	// Tests on the 'Passenger' derived class.
	private void testIdentifiable(){
		Passenger passengerObject = new Passenger();
		System.out.println("Generated any random integer -> " + Passenger.rGen.next());
		System.out.println("Generated any random integer in range 1-1000 -> " + 
						   Passenger.rGen.next(1,1000));
	}
	
	// Auxiliary methods.
	
	// Convert integer selection to enumeration selection.
	private test toTestEnum(int selection) {
		test selected = test.NONE;
		if (selection == 1) 
			selected = test.OBJECT;
		else if (selection == 2)
			selected = test.VEHICLE;
		else if (selection == 3)
			selected = test.BUS;
		else if (selection == 4)
			selected = test.PERSON;
		else if (selection == 5)
			selected = test.DRIVER;
		else if (selection == 6)
			selected = test.PASSENGER;
		else if (selection == 7)
			selected = test.IDENTIFIABLE;
		else if (selection == 9)
			selected = test.EXIT;
		return selected;
	}
} 